import { JupyterFrontEnd } from '@jupyterlab/application';
import { DocumentRegistry } from '@jupyterlab/docregistry';
import { Token } from '@lumino/coreutils';
import { ISignal } from '@lumino/signaling';
import { Panel, Widget } from '@lumino/widgets';
/**
 * The Jupyter Notebook application shell token.
 */
export declare const INotebookShell: Token<INotebookShell>;
/**
 * The Jupyter Notebook application shell interface.
 */
export interface INotebookShell extends NotebookShell {
}
/**
 * The application shell.
 */
export declare class NotebookShell extends Widget implements JupyterFrontEnd.IShell {
    constructor();
    /**
     * A signal emitted when the current widget changes.
     */
    get currentChanged(): ISignal<NotebookShell, void>;
    /**
     * The current widget in the shell's main area.
     */
    get currentWidget(): Widget | null;
    /**
     * Get the top area wrapper panel
     */
    get top(): Widget;
    /**
     * Get the menu area wrapper panel
     */
    get menu(): Widget;
    /**
     * Get the left area handler
     */
    get leftHandler(): SideBarHandler;
    /**
     * Get the right area handler
     */
    get rightHandler(): SideBarHandler;
    /**
     * Is the left sidebar visible?
     */
    get leftCollapsed(): boolean;
    /**
     * Is the right sidebar visible?
     */
    get rightCollapsed(): boolean;
    /**
     * Promise that resolves when the main widget is loaded
     */
    get restored(): Promise<void>;
    /**
     * Activate a widget in its area.
     */
    activateById(id: string): void;
    /**
     * Add a widget to the application shell.
     *
     * @param widget - The widget being added.
     *
     * @param area - Optional region in the shell into which the widget should
     * be added.
     *
     * @param options - Optional open options.
     *
     */
    add(widget: Widget, area?: Shell.Area, options?: DocumentRegistry.IOpenOptions): void;
    /**
     * Collapse the top area and the spacer to make the view more compact.
     */
    collapseTop(): void;
    /**
     * Expand the top area to show the header and the spacer.
     */
    expandTop(): void;
    /**
     * Return the list of widgets for the given area.
     *
     * @param area The area
     */
    widgets(area: Shell.Area): IterableIterator<Widget>;
    /**
     * Expand the left panel to show the sidebar with its widget.
     */
    expandLeft(id?: string): void;
    /**
     * Collapse the left panel
     */
    collapseLeft(): void;
    /**
     * Expand the right panel to show the sidebar with its widget.
     */
    expandRight(id?: string): void;
    /**
     * Collapse the right panel
     */
    collapseRight(): void;
    private _topWrapper;
    private _topHandler;
    private _menuWrapper;
    private _menuHandler;
    private _leftHandler;
    private _rightHandler;
    private _spacer;
    private _main;
    private _currentChanged;
    private _mainWidgetLoaded;
}
/**
 * A namespace for Shell statics
 */
export declare namespace Shell {
    /**
     * The areas of the application shell where widgets can reside.
     */
    type Area = 'main' | 'top' | 'left' | 'right' | 'menu';
}
/**
 * A name space for SideBarPanel functions.
 */
export declare namespace SideBarPanel {
    /**
     * The areas of the sidebar panel
     */
    type Area = 'left' | 'right';
}
/**
 * A class which manages a side bar that can show at most one widget at a time.
 */
export declare class SideBarHandler {
    /**
     * Construct a new side bar handler.
     */
    constructor(area: SideBarPanel.Area);
    /**
     * Get the current widget in the sidebar panel.
     */
    get currentWidget(): Widget | null;
    /**
     * Get the area of the side panel
     */
    get area(): SideBarPanel.Area;
    /**
     * Whether the panel is visible
     */
    get isVisible(): boolean;
    /**
     * Get the stacked panel managed by the handler
     */
    get panel(): Panel;
    /**
     * Get the widgets list.
     */
    get widgets(): Readonly<Widget[]>;
    /**
     * Signal fired when a widget is added to the panel
     */
    get widgetAdded(): ISignal<SideBarHandler, Widget>;
    /**
     * Signal fired when a widget is removed from the panel
     */
    get widgetRemoved(): ISignal<SideBarHandler, Widget>;
    /**
     * Expand the sidebar.
     *
     * #### Notes
     * This will open the most recently used widget, or the first widget
     * if there is no most recently used.
     */
    expand(id?: string): void;
    /**
     * Activate a widget residing in the stacked panel by ID.
     *
     * @param id - The widget's unique ID.
     */
    activate(id: string): void;
    /**
     * Test whether the sidebar has the given widget by id.
     */
    has(id: string): boolean;
    /**
     * Collapse the sidebar so no items are expanded.
     */
    collapse(): void;
    /**
     * Add a widget and its title to the stacked panel.
     *
     * If the widget is already added, it will be moved.
     */
    addWidget(widget: Widget, rank: number): void;
    /**
     * Hide the side panel
     */
    hide(): void;
    /**
     * Show the side panel
     */
    show(): void;
    /**
     * Find the insertion index for a rank item.
     */
    private _findInsertIndex;
    /**
     * Find the index of the item with the given widget, or `-1`.
     */
    private _findWidgetIndex;
    /**
     * Find the widget with the given id, or `null`.
     */
    private _findWidgetByID;
    /**
     * Refresh the visibility of the stacked panel.
     */
    private _refreshVisibility;
    private _onWidgetRemoved;
    private _area;
    private _isHiddenByUser;
    private _items;
    private _panel;
    private _widgetPanel;
    private _currentWidget;
    private _lastCurrentWidget;
    private _widgetAdded;
    private _widgetRemoved;
}
